﻿local B = BardTools
local Elements = {}
B.DropDown = {}

local standardXOffset = 20
local standardYOffset = 9
local standardHeight = 20 + 2 * standardYOffset

local function GetTotalElementHeight(column)
    local height = 9
    
    for k,v in pairs(Elements[column]) do
        height = height + v.height
    end
    
    return height
end

function B:OpenConfig()
    if (not BardToolsAnzuFrameAnchor) then
        B:CreateAnzuFrames()
    end
    
    if (not BardToolsConfig) then
        B:CreateConfig(true)
        BardToolsConfig:Show()
    else
        BardToolsConfig:Show()
    end
    
    B:UpdateAnzuFrameSettings()
end

function B:CreateConfig(dontHide)
    -- Configuration box
    PlaySound("igCharacterInfoOpen")
    local f = CreateFrame("Frame", "BardToolsConfig", UIParent)
    f:SetSize(310, 420)
    f:SetPoint("CENTER")
    f:SetFrameLevel(1)
    f:SetMovable(1)
    f:SetFrameStrata("HIGH")
    f:SetScript("OnMouseDown", function() BardToolsConfig:StartMoving() end)
    f:SetScript("OnMouseUp", function() BardToolsConfig:StopMovingOrSizing() end)
    f:SetScript("OnHide", function() B:UpdateAnzuFrameSettings() end)
    f:SetBackdrop({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\AddOns\\BardTools\\Textures\\ConfigBorder",
        tile = true,
        tileSize = 16,
        edgeSize = 16,
        insets = {left = 5, right = 5, top = 5, bottom = 5}
    })
    tinsert(UISpecialFrames, f:GetName())
    
    -- Configuration box title
    f = CreateFrame("Frame", "BardToolsConfigTitle", BardToolsConfig)
    f:SetHeight(30)
    f:SetWidth(200)
    f:SetPoint("CENTER", "BardToolsConfig", "TOP", 0, -15)
    f:SetScript("OnMouseDown", function() BardToolsConfig:StartMoving() end)
    f:SetScript("OnMouseUp", function() BardToolsConfig:StopMovingOrSizing() end)
    f:SetBackdrop({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\AddOns\\BardTools\\Textures\\ConfigBorder",
        tile = true,
        tileSize = 16,
        edgeSize = 16,
        insets = {left = 5, right = 5, top = 5, bottom = 5}
    })
    f:SetScript("OnShow", function() PlaySound("igCharacterInfoOpen") end)
    f:SetScript("OnHide", function() PlaySound("igCharacterInfoClose") end)
    
    local t = f:CreateFontString()
    t:SetPoint("CENTER", f, "CENTER", 0, 0)
    t:SetFont("Fonts\\FRIZQT__.TTF", 12)
    t:SetText("BardTools - AnzuHelper")
    
    local function CloseButton_OnClick()
        BardToolsConfig:Hide()
    end
    
    B:CreateConfigButton("BardToolsConfigCloseButton", "Close", CloseButton_OnClick, 1)
    
    B:EmptySpace(0)
    B:EmptySpace(2)
    B:CreateConfigLabel("BardToolsConfigHorizontalGrowthLabel", "Horizontal Growth")
    local d = B:CreateConfigDropDown("BardToolsConfigHorizontalGrowthDropDown", B.HorizontalGrowthDropDown, BardToolsData.AnzuFrames.horizontalGrowthDirection, 2)
    
    B:CreateConfigLabel("BardToolsConfigVerticalGrowthLabel", "Vertical Growth")
    local d = B:CreateConfigDropDown("BardToolsConfigVerticalGrowthDropDown", B.VerticalGrowthDropDown, BardToolsData.AnzuFrames.verticalGrowthDirection, 2)
    
    B:CreateConfigLabel("BardToolsConfigGroupTypeLabel", "Group Type")
    local d = B:CreateConfigDropDown("BardToolsConfigGroupTypeDropDown", B.GroupTypeDropDown, BardToolsData.AnzuFrames.groupType, 2)
    
    B:EmptySpace(1,3)
    B:CreateConfigLabel("BardToolsConfigSpacingLabel", "Spacing")
    local c = B:CreateConfigEditbox("BardToolsConfigSpacingEditbox", BardToolsData.AnzuFrames.spacing, 2, BardToolsData.AnzuFrames, "spacing", 68)
    c:SetNumeric(true)
    c:SetMaxLetters(3)
    
    B:CreateConfigLabel("BardToolsConfigWidthLabel", "Frame Width")
    local a = B:CreateConfigEditbox("BardToolsConfigWidthEditbox", BardToolsData.AnzuFrames.width, 2, BardToolsData.AnzuFrames, "width", 68)
    a:SetNumeric(true)
    a:SetMaxLetters(3)
    
    B:CreateConfigLabel("BardToolsConfigHeightLabel", "Frame Height")
    local b = B:CreateConfigEditbox("BardToolsConfigHeightEditbox", BardToolsData.AnzuFrames.height, 2, BardToolsData.AnzuFrames, "height", 68)
    b:SetNumeric(true)
    b:SetMaxLetters(3)
    
    B:CreateConfigLabel("BardToolsConfigDisplayNamesLabel", "Display Names", 0)
    B:CreateConfigCheckButton("BardToolsConfigDisplayNamesCheckButton", "Check to display names on the frames", BardToolsData.AnzuFrames.displayNames, B.DisplayNamesCheckButton_OnClick, 2)
    
    B:CreateConfigLabel("BardToolsInterruptButtonLabel", (BardToolsData.interruptDuty or "No Interrupt Duty"))
    B:CreateConfigButton("BardToolsInterruptButton", "Interrupt", B.InterruptButton_OnClick, 2)
    
    B:CreateConfigLabel("BardToolsDispelButtonLabel", (BardToolsData.dispelDuty or "No Dispel Duty"))
    B:CreateConfigButton("BardToolsDispelButton", "Dispel", B.DispelButton_OnClick, 2)
    
    if (not dontHide) then
        BardToolsConfig:Hide()
    end
end

function B:InterruptButton_OnClick()
    B:BroadcastInterruptDuty()
end

function B:DispelButton_OnClick()
    B:BroadcastDispelDuty()
end

function B:DisplayNamesCheckButton_OnClick()
    BardToolsData.AnzuFrames.displayNames = not BardToolsData.AnzuFrames.displayNames
    B:UpdateAnzuFrameNames()
end

function B:GroupTypeDropDown()
    local function OnClick(self, arg1)
        BardToolsData.AnzuFrames.groupType = arg1
        UIDropDownMenu_SetText(BardToolsConfigGroupTypeDropDown, arg1)
        B:UpdateAnzuFrameSettings()
    end
    
    local info = UIDropDownMenu_CreateInfo()
    local c = BardToolsData.AnzuFrames.groupType
    
    info.func = OnClick
    
    info.text, info.checked, info.arg1 = "Grid", c == "Grid", "Grid"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "Vertical Stack", c == "Vertical Stack", "Vertical Stack"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "Horizontal Stack", c == "Horizontal Stack", "Horizontal Stack"
    UIDropDownMenu_AddButton(info)
end

function B:VerticalGrowthDropDown()
    local function OnClick(self, arg1)
        BardToolsData.AnzuFrames.verticalGrowthDirection = arg1
        UIDropDownMenu_SetText(BardToolsConfigVerticalGrowthDropDown, arg1)
        B:UpdateAnzuFrameSettings()
    end
    
    local info = UIDropDownMenu_CreateInfo()
    local c = BardToolsData.AnzuFrames.verticalGrowthDirection
    
    info.func = OnClick
    
    info.text, info.checked, info.arg1 = "Down", c == "Down", "Down"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "Up", c == "Up", "Up"
    UIDropDownMenu_AddButton(info)
end

function B:HorizontalGrowthDropDown()
    local function OnClick(self, arg1)
        BardToolsData.AnzuFrames.horizontalGrowthDirection = arg1
        UIDropDownMenu_SetText(BardToolsConfigHorizontalGrowthDropDown, arg1)
        B:UpdateAnzuFrameSettings()
    end
    
    local info = UIDropDownMenu_CreateInfo()
    local c = BardToolsData.AnzuFrames.horizontalGrowthDirection
    
    info.func = OnClick
    
    info.text, info.checked, info.arg1 = "Right", c == "Right", "Right"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "Left", c == "Left", "Left"
    UIDropDownMenu_AddButton(info)
end

function B:EmptySpace(column, reps)
    
    
    if (not Elements[column]) then
        Elements[column] = {}
    end
    
    if (type(reps) == "number") then
        for i = 1,reps do
            local f = {}
            f.height = standardHeight
            tinsert(Elements[column], f)
        end
    else
        local f = {}
        f.height = standardHeight
        tinsert(Elements[column], f)
    end
end

function B:CreateConfigLabel(name, text, column)
    local column = column or 0
    local xOffset = standardXOffset + column * 100
    local yOffset = standardYOffset
    local textHeight = 12
    
    if (not Elements[column]) then
        Elements[column] = {}
    end
    
    local f = CreateFrame("Frame", name, BardToolsConfig)
    f:SetPoint("BOTTOMLEFT", BardToolsConfig, "BOTTOMLEFT", xOffset, yOffset + GetTotalElementHeight(column))
    local t = f:CreateFontString("$parentText")
    t:SetFont("Fonts\\FRIZQT__.TTF", textHeight)
    t:SetJustifyH("LEFT")
    t:SetText(text)
    t:SetNonSpaceWrap(false)
    
    f:SetSize(200, textHeight)
    t:SetAllPoints(f)
    
    f.height = standardHeight
    tinsert(Elements[column], f)
end

function B:CreateConfigCheckButton(name, tooltip, checked, script, column)
    local column = column or 0
    local xOffset = standardXOffset + column * 100
    local yOffset = standardYOffset - 4
    local size = 20
    
    if (not Elements[column]) then
        Elements[column] = {}
    end
    
    f = CreateFrame("CheckButton", name, BardToolsConfig, "ChatConfigCheckButtonTemplate")
    f:SetSize(size,size)
    f:SetPoint("BOTTOMLEFT", BardToolsConfig, "BOTTOMLEFT", xOffset, yOffset + GetTotalElementHeight(column))
    f:SetFrameLevel(2)
    f:SetScript("OnClick", function() PlaySound("GAMEABILITYBUTTONMOUSEDOWN") script() end)
    f:SetChecked(checked)
    f:SetNormalTexture("Interface\\AddOns\\BardTools\\Textures\\ConfigCheckboxUp")
    f:SetPushedTexture("Interface\\AddOns\\BardTools\\Textures\\ConfigCheckboxDown")
    f:SetHighlightTexture("Interface\\AddOns\\BardTools\\Textures\\ConfigButtonHighlight",0)
    f:SetCheckedTexture("Interface\\AddOns\\BardTools\\Textures\\ConfigCheckboxChecked")
    f:SetDisabledTexture("Interface\\AddOns\\BardTools\\Textures\\ConfigButtonDisabled")
    f.tooltip = tooltip
    
    f.height = standardHeight
    tinsert(Elements[column], f)
end

function B:CreateConfigEditbox(name, text, column, table, variable, width, height)
    local column = column or 0
    local xOffset = standardXOffset + column * 100
    local yOffset = standardYOffset - 9
    local width = width or 200
    local height = height or 30
    
    if (not Elements[column]) then
        Elements[column] = {}
    end
    
    local f = CreateFrame("Editbox", name, BardToolsConfig)
    f:SetFontObject(GameFontNormal)
    f:SetSize(width, height)
    f:SetPoint("BOTTOMLEFT", BardToolsConfig, "BOTTOMLEFT", xOffset, yOffset + GetTotalElementHeight(column))
    f:SetFrameLevel(2)
    f:SetBackdrop({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\AddOns\\BardTools\\Textures\\ConfigBorder",
        tile = true,
        tileSize = 16,
        edgeSize = 16,
        insets = {left = 5,right = 5,top = 5,bottom = 5}
    })
    f:SetAutoFocus(false)
    f:ClearFocus()
    --f:SetFont("Fonts\\FRIZQT__.TTF", 12)
    f:SetTextColor(1,1,1)
    f:Insert(text)
    f:SetMaxLetters(50)
    f:SetTextInsets(9,0,0,0)
    
    local function OnEditFocusLost()
        local text = tonumber(f:GetText()) -- Returns nil if text is empty or contains non-numeric characters
        if (text) then
            table[variable] = text
            B:UpdateAnzuFrameSettings()
        else
            f:SetText(table[variable])
        end
        f:HighlightText(0,0)
    end
    
    local function OnEscapePressed()
        f:SetText(table[variable])
        f:ClearFocus()
    end
    
    f:SetScript("OnEscapePressed", OnEscapePressed)
    f:SetScript("OnEnterPressed", function() f:ClearFocus() end)
    f:SetScript("OnEditFocusGained", function() f:HighlightText() end)
    f:SetScript("OnEditFocusLost", OnEditFocusLost)
    
    f.height = standardHeight
    tinsert(Elements[column], f)
    
    return f
end

function B:NameEditboxOnTabPressed()
    local numEditboxes = 2
    for i=1,numEditboxes do
        f = _G["BardToolsConfigEditbox" .. i]
        if (f and f:HasFocus()) then
            if (i < numEditboxes) then
                _G["BardToolsConfigEditbox" .. i+1]:SetFocus()
                break
            else
                BardToolsConfigEditbox1:SetFocus()
                break
            end
        end
    end
end

function B:CreateConfigButton(name, text, script, column, width)
    local column = column or 0
    local xOffset = standardXOffset + column * 100
    local yOffset = standardYOffset - 3
    local width = width or 70
    local height = 20
    
    if (not Elements[column]) then
        Elements[column] = {}
    end
    
    f = CreateFrame("Button", name, BardToolsConfig)
    if (text ~= "Close") then
        f:SetScript("OnClick", function() PlaySound("GAMEABILITYBUTTONMOUSEDOWN") script() end)
    else
        f:SetScript("OnClick", script) -- Let's not have the close button play a sound other than the "close" sound...
    end
    f:SetHeight(height)
    f:SetWidth(width)
    f:SetParent(BardToolsConfig)
    f:SetPoint("BOTTOMLEFT", BardToolsConfig, "BOTTOMLEFT", xOffset, yOffset + GetTotalElementHeight(column))
    f:SetFrameLevel(2)
    f:SetNormalTexture("Interface\\AddOns\\BardTools\\Textures\\ConfigButtonUp")
    f:SetPushedTexture("Interface\\AddOns\\BardTools\\Textures\\ConfigButtonDown")
    f:SetHighlightTexture("Interface\\AddOns\\BardTools\\Textures\\ConfigButtonHighlight", 0)
    f:SetDisabledTexture("Interface\\AddOns\\BardTools\\Textures\\ConfigButtonDisabled")
    f.tooltip = tooltip
    
    local font = f:CreateFontString()
    f:SetFontString(font)
    f:SetPushedTextOffset(-1, -1)
    font:SetFont("Fonts\\FRIZQT__.TTF", height * 0.6)
    font:SetText(text)
    font:SetTextColor(0, 0, 0)
    
    f.height = standardHeight
    tinsert(Elements[column], f)
end

function B:PugModeDropDown()
    local function OnClick(self, arg1)
        BDB.pugModeChannel = arg1
        UIDropDownMenu_SetText(BardToolsPUGModeChannelDropDown, arg1)
    end
    
    local info = UIDropDownMenu_CreateInfo()
    local c = BDB.pugModeChannel
    
    info.func = OnClick
    
    info.text, info.checked, info.arg1 = "SAY", c == "SAY", "SAY"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "YELL", c == "YELL", "YELL"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "PARTY", c == "PARTY", "PARTY"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "RAID", c == "RAID", "RAID"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "RAID_WARNING", c == "RAID_WARNING", "RAID_WARNING"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "WHISPER", c == "WHISPER", "WHISPER"
    UIDropDownMenu_AddButton(info)
    
    info.text, info.checked, info.arg1 = "INSTANCE_CHAT", c == "INSTANCE_CHAT", "INSTANCE_CHAT"
    UIDropDownMenu_AddButton(info)
end

function B:CreateConfigDropDown(name, script, text, column)
    local column = column or 0
    local xOffset = standardXOffset + column * 100 - 35
    local yOffset = standardYOffset - 13
    local height = 25
    
    if (not Elements[column]) then
        Elements[column] = {}
    end
    
    local f = CreateFrame("Frame", name, BardToolsConfig, "UIDropDownMenuTemplate")
    f:SetPoint("BOTTOMLEFT", BardToolsConfig, "BOTTOMLEFT", xOffset, yOffset + GetTotalElementHeight(column))
    UIDropDownMenu_SetWidth(f, 80)
    UIDropDownMenu_Initialize(f, script)
    UIDropDownMenu_SetText(f, text)
    
    f.height = standardHeight
    tinsert(Elements[column], f)
end

function round(num, dec)
    local number = string.format("%." .. dec .. "f", tostring(num))
    return tonumber(number)
end

function roundString(num,dec)
    return string.format("%." .. dec .. "f", tostring(num))
end
